<?php
/**
 * @package	Hikashop - Variants Quantity Synch
 * @version	1.0.2
 * @author	HikaShop
 * @copyright	(C) 2010-2023 HIKARI SOFTWARE. All rights reserved.
 * @license	GNU/GPLv3 http://www.gnu.org/licenses/gpl-3.0.html
 */
defined('_JEXEC') or die('Restricted access');
?><?php
class plgHikashopVariants_quantity_synch extends JPlugin
{
	function __construct(&$subject, $config){
		parent::__construct($subject, $config);
	}

	function onBeforeProductDelete(&$ids, &$do){
		$this->parents = array();
		if(empty($ids) || !is_array($ids) || count($ids) < 1) {
			return;
		}
		$db = JFactory::getDBO();
		$db->setQuery('SELECT DISTINCT product_parent_id FROM #__hikashop_product WHERE product_id IN ('.implode(',', $ids).') AND product_parent_id > 0');
		$elements = $db->loadObjectList('product_parent_id');
		$this->parents = array_keys($elements);
	}
	function onAfterProductDelete(&$ids){
		if(empty($this->parents) || !count($this->parents))
			return;
		foreach($this->parents as $product_parent_id) {
			$this->_updateProduct($product_parent_id);
		}
	}
	function onAfterProductCreate(&$product){
		$this->_processProduct($product, true);
	}
	function onAfterProductUpdate(&$product){
		$this->_processProduct($product, false);
	}
	function _processProduct(&$product, $new) {

		if(empty($product->product_id))
			return;
		$class = hikashop_get('class.product');
		if(empty($product->product_type)) {
			$data = $class->get($product->product_id);
			$type = $data->product_type;
		} else {
			$type = $product->product_type;
		}
		if($type == 'main') {
			if(isset($product->product_quantity) && $product->product_quantity != -1) {
				static $done = false;
				if(!$done) {
					$done = true;
					$this->_updateProduct($product->product_id);
				}
			}
			return;
		} elseif($type != 'variant') {
			return;
		}

		if(empty($product->product_parent_id)) {
			$data = $class->get($product->product_id);
			$product_parent_id = $data->product_parent_id;
		} else {
			$product_parent_id = $product->product_parent_id;
		}

		$parent = $class->get($product_parent_id);

		if(empty($parent) || (int)$parent->product_quantity == -1)
			return;

		$this->_updateProduct($product_parent_id);
	}
	function _updateProduct($product_parent_id) {
		$db = JFactory::getDBO();
		$db->setQuery('SELECT sum(product_quantity) FROM #__hikashop_product WHERE product_parent_id = '.(int)$product_parent_id.' AND product_type=\'variant\' AND product_quantity != -1');
		$newQuantity = $db->loadResult();

		$parentSave = new stdClass();
		$parentSave->product_id = $product_parent_id;
		$parentSave->product_type = 'main';
		$parentSave->product_quantity = $newQuantity;

		$class = hikashop_get('class.product');
		if(method_exists($class, 'saveRaw'))
			$class->saveRaw($parentSave);
		else
			$class->save($parentSave);
	}
}
